/* This is the system class for J-Offline. It implements a few functions
   which interact with the underlying system - the OS or the Java Platform.

   The code is not object-orientated - it is merely a bunch of functions
   factored out.

   (c) 2003 Ben Bucksch <http://www.bucksch.org>.
   New-BSD-style license.
*/

package org.bucksch.tools.joffline;

import java.io.*;
import java.net.*;
import java.util.Enumeration;
import java.util.Date;

class SysCall
{
  public SysCall()
  {
  }

  /* Check (using |ping|), if the Internet is reachable.
     Current impl. requires 0% loss to all hosts.
     Blocking.
     @param iface   See |ping|.
     @return
  */
  static public boolean internetWorks(String iface)
  {
    if (iface.length() > 0 && !ifaceIsUp(iface))
      // Somebody broke the internet! Call the police!
      // Make sure to complain at internet.com as well.
      // But I bet the music industry had a hand in that.
      return false;

    final boolean noloss = false;
    final int pings = 1;
    int nonreach = 0;
    if (!ping("www.yourisp.invalid", iface, pings, noloss)) nonreach++;
    if (!ping("www.yourserver.invalid", iface, pings, noloss)) nonreach++;
    if (!ping("www.yourpeer.invalid", iface, pings, noloss)) nonreach++;
    return nonreach <= 1; // allow one server down / ping to fail

    /* zero tolerance implementation
    final boolean noloss = false;
    if (!ping("www.yourisp.invalid", iface, pings, noloss)) return false;
    if (!ping("www.yourserver.invalid", iface, pings, noloss)) return false;
    if (!ping("www.yourpeer.invalid", iface, pings, noloss)) return false;
    return true;
    */
  }

  /* Send an ICMP ping to |server| and check the response.
     Blocking.
     @param iface   Use this local network interface. If "",use normal routing.
                    Warning! This is ignored by the |ping| OS programs I tried.
                    They just use the normal routing.
     @param numebrOfPings   ping that often in sequence
     @param perfect   0% loss required
     @return   If server was reachable
  */
  static public boolean ping(String server, String iface,
                             int numberOfPings, boolean perfect)
  {
    String command = "/bin/ping -c " + numberOfPings + " -n ";
    if (iface.length() > 0)
    {
      //log("looking up bind ip address for interface " +iface);
      try
      {
        NetworkInterface ni = NetworkInterface.getByName(iface);
        Enumeration en = ni.getInetAddresses();
        if (!en.hasMoreElements())
          return false;
        InetAddress ia;
        ia = (InetAddress) en.nextElement();
        command += "-I " + ia.getHostAddress() + " ";
      }
      catch (java.net.SocketException e)
      {
        return false;
      }
      catch (java.util.NoSuchElementException e)
      {
        return false;
      }
    }
    command += server;
    boolean ret = false;
    if (perfect)
    {
      String output = executeWithOutput(command);
      //return output.matches(".* 0% packet loss.*");
      ret = output.matches(".* 0% packet loss.*");
    }
    else
    {
      //return executeWithExitValue(command) == 0;
      ret = executeWithExitValue(command) == 0;
    }
    //log("ping to " + server + (ret ? " succeeded." : " failed."));
    if (!ret)
      log("ping to " + server + " failed.");
    return ret;
  }

  /* asks ifconfig about network interface |iface|.
     @return   iface is reported as "UP" by OS
  */
  static public boolean ifaceIsUp(String iface)
  {
    String output = executeWithOutput("/sbin/ifconfig " + iface);
    return output.matches(".* UP .*");
  }


  /* Dials out or closes the link.
     Does not block until the line is up/down
     @param dial   if true, dial out. if false, close an existing link
  */
  static public void ifaceDialClose(String iface, boolean dial)
  {
    if (iface == "ppp0")
    {
      if (dial)
      {
        executeWithExitValue("/etc/init.d/ppp restart");
      }
      else // close link
      {
        executeWithExitValue("/etc/init.d/ppp stop");
      }
    }
    else if (iface == "ippp0")
    {
      if (dial)
      {
        executeWithExitValue("sh /etc/isdn/isp/changeisp.sh arcor");
      }
      else // close link
      {
        executeWithExitValue("sh /etc/isdn/isp/changeisp.sh void");
      }
    }
  }

  static public void setDefaultRoute(String iface)
  {
    executeWithExitValue("route del default");
    executeWithExitValue("route add default " + iface);
    log("set default route to " + iface);
    ipChanged(iface);
  }

  static protected void ipChanged(String new_iface)
  {
    // advertize new ip address
    log("advertizing new IP address using (changed) normal routing");
    execute("sh /etc/ppp/ip-up.d/send-current-ip", 30);
    log("done advertizing new IP address");
  }



  /* Executes a native command.
     Blocks until the command finished.
     @return   Output of the command
  */
  static public String executeWithOutput(String command)
  {
    String output = "";
    try
    {
      Process proc = execute(command, -1);
      DataInputStream istream = new DataInputStream(proc.getInputStream());
      //DataOutputStream os = new DataOutputStream(proc.getOutputStream());

      String istring;
      while (istream.available() > 0)
      {
        istring = istream.readLine();
        output += istring;
        //log("cmdoutput: " + istring);
      }
    }
    catch(java.io.IOException e)
    {
      log("IOException");
    }
    //log("done");
    return output;
  }

  /* Executes a native command.
     Blocks until the command finished.
     @return   Exit value return by the command.
               -1, if exception occured.
               -2, if the process didn't finish in time. - not implemented
  */
  static public int executeWithExitValue(String command)
  {
    return execute(command, -1).exitValue();
    //int exitval = -1;
    //return exitval;
  }

  /* Executes a native command.
     Blocks until the command finished.
     @param waitSeconds   if -1, wait indefinitely until the process finished
                          if => 0, always wait |waitSeconds| for the process to
                             finish, and if it's not finished by then, kill it.
                             - not implemented
     @return   Output of the command
  */
  static public Process execute(String command, int waitSeconds)
  {
    Process proc = null; // will cause null pointer exceptions in error cases
    try
    {
      //log("executing command " + command); good for debugging
      proc = Runtime.getRuntime().exec(command);
      //log("executed, waiting for process to finish");
      if (waitSeconds == -1)
      {
        proc.waitFor();
      }
      else
      {
        sleep(waitSeconds);
        proc.exitValue(); // will throw exception, if still running
      }
      //log("process finished");
    }
    catch(java.lang.IllegalThreadStateException e)
      // thrown by exitValue(), if the process is not finished at that point
    {
      log("process " + command + " ran longer than allowed, killing it");
      proc.destroy();
    }
    catch(java.io.IOException e)
    {
      log("IOException");
    }
    catch(java.lang.InterruptedException e)
    {
      log("InterruptedException");
    }
    return proc;
  }

  static public void sleep(int seconds)
  {
    try
    {
      Thread.currentThread().sleep(seconds * 1000);
    }
    catch(java.lang.InterruptedException e)
    {
      log("InterruptedException");
    }
  }

  /* current impl. outputs |message| to stdout, with a timestamp
   */
  static public void log(String message)
  {
    System.out.println(new Date().toLocaleString() + " " + message);
  }
}
